@@ -11,11 +11,13 @@
 # under the License.
 
 import contextlib
+import logging
 
 import keystoneauth1.exceptions.http as ks_exceptions
 import osc_lib.exceptions as exceptions
 import simplejson as json
-import six
+
+from osc_placement import version
 
 
 _http_error_to_exc = {
@@ -24,6 +26,9 @@ _http_error_to_exc = {
 }
 
 
+LOG = logging.getLogger(__name__)
+
+
 @contextlib.contextmanager
 def _wrap_http_exceptions():
     """Reraise osc-lib exceptions with detailed messages."""
@@ -36,7 +41,7 @@ def _wrap_http_exceptions():
             msg = detail.split('\n')[-1].strip()
             exc_class = _http_error_to_exc.get(exc.http_status,
                                                exceptions.CommandError)
-            six.raise_from(exc_class(exc.http_status, msg), exc)
+            raise exc_class(exc.http_status, msg) from exc
         else:
             raise
 
@@ -45,7 +50,7 @@ class SessionClient(object):
     def __init__(self, session, ks_filter, api_version='1.0'):
         self.session = session
         self.ks_filter = ks_filter
-        self.api_version = api_version
+        self.negotiate_api_version(api_version)
 
     def request(self, method, url, **kwargs):
         version = kwargs.pop('version', None)
@@ -60,3 +65,22 @@ class SessionClient(object):
                                         headers=headers,
                                         endpoint_filter=self.ks_filter,
                                         **kwargs)
+
+    def negotiate_api_version(self, api_version):
+        """Set api_version to self.
+
+        If negotiate version (only majorversion) is given, talk to server to
+        pick up max microversion supported both by client and by server.
+        """
+        if api_version not in version.NEGOTIATE_VERSIONS:
+            self.api_version = api_version
+            return
+        client_ver = version.MAX_VERSION_NO_GAP
+        self.api_version = client_ver
+        resp = self.request('GET', '/', raise_exc=False)
+        if resp.status_code == 406:
+            server_ver = resp.json()['errors'][0]['max_version']
+            self.api_version = server_ver
+            LOG.debug('Microversion %s not supported in server. '
+                      'Falling back to microversion %s',
+                      client_ver, server_ver)
