@@ -14,8 +14,6 @@ import collections
 import copy
 import uuid
 
-import six
-
 from osc_placement.tests.functional import base
 
 
@@ -27,17 +25,24 @@ class TestInventory(base.BaseTestCase):
 
     def test_inventory_show(self):
         rp_uuid = self.rp['uuid']
-        expected = {'min_unit': 1,
-                    'max_unit': 12,
-                    'reserved': 0,
-                    'step_size': 1,
-                    'total': 12,
-                    'allocation_ratio': 16.0}
+        updates = {
+            'min_unit': 1,
+            'max_unit': 12,
+            'reserved': 0,
+            'step_size': 1,
+            'total': 12,
+            'allocation_ratio': 16.0,
+        }
+        expected = updates.copy()
+        expected['used'] = 0
+        args = ['VCPU:%s=%s' % (k, v) for k, v in updates.items()]
 
-        args = ['VCPU:%s=%s' % (k, v) for k, v in expected.items()]
         self.resource_inventory_set(rp_uuid, *args)
-        self.assertEqual(expected,
-                         self.resource_inventory_show(rp_uuid, 'VCPU'))
+
+        self.assertEqual(
+            expected,
+            self.resource_inventory_show(rp_uuid, 'VCPU', include_used=True),
+        )
 
     def test_inventory_show_not_found(self):
         rp_uuid = self.rp['uuid']
@@ -46,7 +51,28 @@ class TestInventory(base.BaseTestCase):
                                 self.resource_inventory_show,
                                 rp_uuid, 'VCPU')
         self.assertIn('No inventory of class VCPU for {}'.format(rp_uuid),
-                      six.text_type(exc))
+                      str(exc))
+
+    def test_inventory_list(self):
+        rp_uuid = self.rp['uuid']
+        updates = {
+            'min_unit': 1,
+            'max_unit': 12,
+            'reserved': 0,
+            'step_size': 1,
+            'total': 12,
+            'allocation_ratio': 16.0,
+        }
+        expected = [updates.copy()]
+        expected[0]['resource_class'] = 'VCPU'
+        expected[0]['used'] = 0
+        args = ['VCPU:%s=%s' % (k, v) for k, v in updates.items()]
+
+        self.resource_inventory_set(rp_uuid, *args)
+
+        self.assertEqual(
+            expected, self.resource_inventory_list(rp_uuid, include_used=True),
+        )
 
     def test_inventory_delete(self):
         rp_uuid = self.rp['uuid']
@@ -58,14 +84,14 @@ class TestInventory(base.BaseTestCase):
                                 self.resource_inventory_show,
                                 rp_uuid, 'VCPU')
         self.assertIn('No inventory of class VCPU for {}'.format(rp_uuid),
-                      six.text_type(exc))
+                      str(exc))
 
     def test_inventory_delete_not_found(self):
         exc = self.assertRaises(base.CommandException,
                                 self.resource_inventory_delete,
                                 self.rp['uuid'], 'VCPU')
         self.assertIn('No inventory of class VCPU found for delete',
-                      six.text_type(exc))
+                      str(exc))
 
     def test_delete_all_inventories(self):
         # Negative test to assert command failure because
@@ -81,7 +107,7 @@ class TestSetInventory(base.BaseTestCase):
         exc = self.assertRaises(
             base.CommandException,
             self.openstack, 'resource provider inventory set')
-        self.assertIn(base.ARGUMENTS_MISSING, six.text_type(exc))
+        self.assertIn(base.ARGUMENTS_MISSING, str(exc))
 
     def test_set_empty_inventories(self):
         rp = self.resource_provider_create()
@@ -93,30 +119,30 @@ class TestSetInventory(base.BaseTestCase):
         exc = self.assertRaises(base.CommandException,
                                 self.resource_inventory_set,
                                 rp['uuid'], 'VCPU')
-        self.assertIn('must have "name=value"', six.text_type(exc))
+        self.assertIn('must have "name=value"', str(exc))
         exc = self.assertRaises(base.CommandException,
                                 self.resource_inventory_set,
                                 rp['uuid'], 'VCPU==')
-        self.assertIn('must have "name=value"', six.text_type(exc))
+        self.assertIn('must have "name=value"', str(exc))
         exc = self.assertRaises(base.CommandException,
                                 self.resource_inventory_set,
                                 rp['uuid'], '=10')
-        self.assertIn('must be not empty', six.text_type(exc))
+        self.assertIn('must be not empty', str(exc))
         exc = self.assertRaises(base.CommandException,
                                 self.resource_inventory_set,
                                 rp['uuid'], 'v=')
-        self.assertIn('must be not empty', six.text_type(exc))
+        self.assertIn('must be not empty', str(exc))
 
         # unknown class
         exc = self.assertRaises(base.CommandException,
                                 self.resource_inventory_set,
                                 rp['uuid'], 'UNKNOWN_CPU=16')
-        self.assertIn('Unknown resource class', six.text_type(exc))
+        self.assertIn('Unknown resource class', str(exc))
         # unknown property
         exc = self.assertRaises(base.CommandException,
                                 self.resource_inventory_set,
                                 rp['uuid'], 'VCPU:fake=16')
-        self.assertIn('Unknown inventory field', six.text_type(exc))
+        self.assertIn('Unknown inventory field', str(exc))
 
     def test_set_multiple_classes(self):
         rp = self.resource_provider_create()
@@ -150,7 +176,7 @@ class TestSetInventory(base.BaseTestCase):
         exc = self.assertRaises(base.CommandException,
                                 self.resource_inventory_set,
                                 rp['uuid'], 'VCPU=8', 'UNKNOWN=4')
-        self.assertIn('Unknown resource class', six.text_type(exc))
+        self.assertIn('Unknown resource class', str(exc))
         self.assertEqual([], self.resource_inventory_list(rp['uuid']))
 
     def test_replace_previous_values(self):
@@ -177,24 +203,24 @@ class TestSetInventory(base.BaseTestCase):
         exc = self.assertRaises(
             base.CommandException,
             self.openstack, 'resource provider inventory class set')
-        self.assertIn(base.ARGUMENTS_MISSING, six.text_type(exc))
+        self.assertIn(base.ARGUMENTS_MISSING, str(exc))
         exc = self.assertRaises(
             base.CommandException,
             self.openstack, 'resource provider inventory class set fake_uuid')
-        self.assertIn(base.ARGUMENTS_MISSING, six.text_type(exc))
+        self.assertIn(base.ARGUMENTS_MISSING, str(exc))
         exc = self.assertRaises(
             base.CommandException,
             self.openstack,
             ('resource provider inventory class set '
              'fake_uuid fake_class --total 5 --unknown 1'))
-        self.assertIn('unrecognized arguments', six.text_type(exc))
+        self.assertIn('unrecognized arguments', str(exc))
         # Valid RP UUID and resource class, but no inventory field.
         rp = self.resource_provider_create()
         exc = self.assertRaises(
             base.CommandException, self.openstack,
             'resource provider inventory class set %s VCPU' % rp['uuid'])
         self.assertIn(base.ARGUMENTS_REQUIRED % '--total',
-                      six.text_type(exc))
+                      str(exc))
 
     def test_set_inventory_for_resource_class(self):
         rp = self.resource_provider_create()
@@ -309,12 +335,36 @@ class TestAggregateInventory(base.BaseTestCase):
     VERSION = '1.3'
 
     def _test_dry_run(self, agg, rps, old_inventories, amend=False):
-        self.resource_inventory_set(
-            agg,
-            'VCPU:allocation_ratio=5.0',
-            'MEMORY_MB:allocation_ratio=6.0',
-            'DISK_GB:allocation_ratio=7.0',
-            aggregate=True, amend=amend, dry_run=True)
+        new_resources = ['VCPU:allocation_ratio=5.0',
+                         'MEMORY_MB:allocation_ratio=6.0',
+                         'DISK_GB:allocation_ratio=7.0']
+        resp = self.resource_inventory_set(
+            agg, *new_resources, aggregate=True, amend=amend, dry_run=True)
+        # Use empty dict to get expected values for full replacement
+        inventories = old_inventories if amend else [{}] * len(old_inventories)
+        # A list of dict keyed by resource class of dict of inventories
+        # Each list element corresponds to one resource provider
+        new_inventories = self._get_expected_inventories(
+            inventories, new_resources)
+        # To compare with actual result, reformat new_inventories to
+        # a dict, keyed by resource provider uuid, of dict of inventories
+        # keyed by resource class
+        expected = {}
+        for rp, inventory in zip(rps, new_inventories):
+            for rc, inv in inventory.items():
+                inv['resource_provider'] = rp['uuid']
+                # For full replacement (not amend) case these values should
+                # be set to empty string.
+                for key in ('max_unit', 'min_unit', 'reserved', 'step_size',
+                            'total', 'reserved', 'step_size'):
+                    if key not in inv:
+                        inv[key] = ''
+            expected[rp['uuid']] = inventory
+        # Reformat raw response (list of inventories) as well
+        resp_dict = collections.defaultdict(dict)
+        for row in resp:
+            resp_dict[row['resource_provider']][row['resource_class']] = row
+        self.assertEqual(expected, resp_dict)
         # Verify the inventories weren't changed (--dry-run)
         for i, rp in enumerate(rps):
             resp = self.resource_inventory_list(rp['uuid'])
@@ -377,7 +427,7 @@ class TestAggregateInventory(base.BaseTestCase):
                                 'VCPU=8',
                                 aggregate=True)
         self.assertIn('No resource providers found in aggregate with uuid {}'
-                      .format(nonexistent_agg), six.text_type(exc))
+                      .format(nonexistent_agg), str(exc))
 
     def test_with_aggregate_one_fails(self):
         # Set up some existing inventories with two resource providers
@@ -399,7 +449,7 @@ class TestAggregateInventory(base.BaseTestCase):
                                 self.resource_inventory_set,
                                 agg, *new_resources, aggregate=True)
         self.assertIn('Failed to set inventory for 1 of 2 resource providers.',
-                      six.text_type(exc))
+                      str(exc))
         output = self.output.getvalue() + self.error.getvalue()
         self.assertIn('Failed to set inventory for resource provider %s:' %
                       rp1_uuid, output)
